#include <xbasic_types.h>

#include "xparameters.h"
#include "stdio.h"
#include "xutil.h"
#include "xstatus.h"

#include <xiic_l.h>
#include <math.h>
/* #include <xgpio.h> */

/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// Global Parameters //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

// Global defines

//#define printf xil_printf

// Controller Defines

#define REPEAT_THRESHOLD 1000

// Push Buttons

#define PUSH_BUTTONS 0x1F /* 0'b11111 */

// Video decoder

#define DECODER_ADDR 0x20 //Read: 0x41, Write: 0x40
#define RED_MASK   0x00003F00
#define GREEN_MASK 0x003F0000
#define BLUE_MASK  0x3F000000

#define RED_SHIFT  8
#define GREEN_SHIFT 16
#define BLUE_SHIFT 24

#define GET_RED_VALUE(raw) (((raw) & RED_MASK) >> RED_SHIFT)
#define GET_GREEN_VALUE(raw) (((raw) & GREEN_MASK) >> GREEN_SHIFT)
#define GET_BLUE_VALUE(raw) (((raw) & BLUE_MASK) >> BLUE_SHIFT)

// Cursor Finder 
// IMPORTANT: manually keep this in sync with cursor_finder_0 instance parameters 

#define C_VIDEO_RAM_BASEADDR          0x40000000
#define C_CURSOR_COLOR_RANGE_BASEADDR 0x42000000
#define C_CURSOR_LOCATION_BASEADDR    0x42001000

// Comparison Module

#define COMMAND_OFFSET 0x08000000
#define X_OFFSET 0x00020000
#define Y_OFFSET 0x00000100
#define VALUE 0x00000001

#define COMP_COMMAND_OFFSET  27
#define COMP_X_OFFSET        17
#define COMP_Y_OFFSET        8
#define COMP_VALUE_OFFSET    0

#define COMP_COMMAND_WRITE_RAW_TEST_BUFFER     1  // write to 640 x 480 x 1 buffer, no ack
#define COMP_COMMAND_WRITE_SCALED_TEST_BUFFER  2  // write to 256 x 256 x 1 buffer, no ack
#define COMP_COMMAND_WRITE_REF_BUFFER          3  // write to 256 x 256 x 8 buffer, no ack
#define COMP_COMMAND_CLEAR_RAW_TEST_BUFFER     4  // clear 640 x 480 x 1 buffer, ack
#define COMP_COMMAND_CLEAR_SCALED_TEST_BUFFER  5  // clear 256 x 256 x 1 buffer, ack
#define COMP_COMMAND_CLEAR_REF_BUFFER          6  // clear 256 x 256 x 8 buffer, ack
#define COMP_COMMAND_COMPARISON_EXECUTE        9  // tells comparison module to do scaling on raw test buffer and compare to ref, ack

// command 19 does scaling only, obsolete
// command 20 used by comparison module to ack when an async command is done

#define COMPARISON_INPUT_REG  0x08000000
#define COMPARISON_OUTPUT_REG 0x08000004
#define COMPARISON_RESET_REG  0x08000010

/////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////// Global Variables //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

/* unsigned int* comparison_module_input_reg = (unsigned int*)COMPARISON_INPUT_REG; */
/* unsigned int* comparison_module_output_reg = (unsigned int*)COMPARISON_OUTPUT_REG; */
/* unsigned int* comparison_module_reset_reg = (unsigned int*)COMPARISON_RESET_REG; */

/* unsigned int* comparison_module_input_reg =  0x08000000; */
/* unsigned int* comparison_module_output_reg = 0x08000004; */
/* unsigned int* comparison_module_reset_reg =  0x08000010; */

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// VIDEO SETUP /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

//Composite Configuration constants
struct VideoModule {
  Xuint8 addr;
  Xuint8 config_val;
  Xuint8 actual_val;
};

#define DECODER_COMP_CONFIG_CNT 18
struct VideoModule decoder_comp[] = { 
  { 0x00, 0x04, 0 },
  { 0x15, 0x00, 0 },
  { 0x17, 0x41, 0 },
  { 0x27, 0x58, 0 },
  { 0x3a, 0x16, 0 }, 
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x50, 0x20, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xd7, 0xea, 0 },
  { 0xe4, 0x3e, 0 },
  { 0xea, 0x0f, 0 }, 
  { 0x0e, 0x00, 0 } };

#define DECODER_CMPNT_CONFIG_CNT 13
struct VideoModule decoder_cmpnt[] = { 
  { 0x00, 0x0a, 0 },
  { 0x27, 0xd8, 0 },
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xe4, 0x3e, 0 },
  { 0x0e, 0x00, 0 } };

void configDecoder(struct VideoModule *decoder, int config_cnt );

void configDecoder(struct VideoModule *decoder, int config_cnt )
{
  Xuint16 send_cnt, i;
  Xuint8 send_data[2] = {0};
  Xuint8 success = 1;
  send_cnt = 2;
//  xil_printf("  Configuring Decoder...\r\n");
  //read in configuration constants and send them to the VDEC one by one
  for( i = 0; i < config_cnt; i++ )
    {

      send_data[0] = decoder[i].addr;

      send_data[1] = decoder[i].config_val;

      send_cnt = XIic_Send(XPAR_IIC_BASEADDR, DECODER_ADDR, send_data, 2,XIIC_STOP );

      if( send_cnt != 2 )
        {
  //        xil_printf("Error writing to address %02x\r\n", decoder[i].addr);
          success = 0;
          break;
        }
    }

  if( success ){}
 //   xil_printf("SUCCESS!\r\n");

}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// DRAWING /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

//Writes the given pixel value to the given screen at the given x-y location
void write_pixel(unsigned int value, unsigned int x, unsigned int y)
{
  ( (unsigned int *) C_VIDEO_RAM_BASEADDR)[(y * 1024) + x] = value;
}

void clear_screen()
{
  int x;
  int y;
  for ( y = 0; y < 480; y++)
    {
      for ( x = 0; x < 640; x++)
        {
          write_pixel(0, x, y); // draw black screen
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// COMPARISON MODULE INTERFACE ////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

#define write_to_comparison_module(reg, command, value, x, y)   \
  *(reg) = (unsigned int)(((command) << COMP_COMMAND_OFFSET) | ((x) << COMP_X_OFFSET) | \
            ((y) << COMP_Y_OFFSET) | ((value) << COMP_VALUE_OFFSET))

#define simple_write_to_comparison_module(reg, value) \
  *(reg) = (unsigned int)(value)

void wait( int wait_time )
{
  int i;
  for ( i = 0; i < wait_time; i++ ){}
  
  return;
}

// Wait until done
int wait_until_done(int* poll_address,int* ak_address)
{
	int a;
	int value = *poll_address;
	while ( ( value & 0xff000000 ) != 0xaa000000 )
		value = *poll_address;

	*ak_address = (0xa0000000);
	return ( value & 0x00ffffff );
};

// Gaussian distribution LUT
int gaussian( int delta )
{
  if ( delta < 1 )
    return 255;
  else if ( delta < 2 )
    return 250;
  else if ( delta < 3 )
    return 245;
  else if ( delta < 5 )
    return 230;
  else if ( delta < 7 )
    return 215;
  else if ( delta < 9 )
    return 190;
  else if ( delta < 11 )
    return 170;
  else if ( delta < 14 )
    return 150;
  else if ( delta < 17 )
    return 125;
  else if ( delta < 20 )
    return 100;
  else if ( delta < 24 )
    return 90;
  else if ( delta < 28 )
    return 80;
  else if ( delta < 33 )
    return 70;
  else if ( delta < 38 )
    return 60;
  else if ( delta < 45 )
    return 50;
  else if ( delta < 52 )
    return 40;
  else
    return 0;
}

// Writes a horizontal line
void write_1( unsigned int* address_register)
{
  int x_value, y_value, value;
  for ( y_value = 0; y_value < 256; y_value++ )
    {	
      for ( x_value = 0; x_value < 256; x_value++ )
        {
          value = gaussian(abs( 127- x_value ));
          if ( value >= 0 )
            {
              *address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(value*VALUE);
            }
          else
            {
              *address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(0*VALUE);
            }
        }
    }
}
// Writes a vertical line
void write_2( unsigned int* address_register)
{
  int x_value, y_value, value;
  for ( y_value = 0; y_value < 256; y_value++ )
    {	
      for ( x_value = 0; x_value < 256; x_value++ )
        {
          value = gaussian(abs( 127 - y_value ));
          *address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(value*VALUE);
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// MAIN ////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

#define GPO_REG_OFFSET 0x124

#define SEND_CNT 3
#define RECV_CNT 3
#define GPO_RESETS_OFF 1
#define GPO_RESET_IIC 3
#define GPO_RESET_DECODER 0

int main() 
{

  XStatus status;
  int x, y;

  int* comparison_module_input_reg =  0x08000000;
  int* comparison_module_output_reg = 0x08000004;
  int* comparison_module_reset_reg =  0x08000010;
  *comparison_module_reset_reg = 0x00000000;
  *comparison_module_reset_reg = 0x00000001;
  /* ////////////////////////////// Initializations ////////////////////////////// */

/*
  configDecoder(decoder_comp, DECODER_COMP_CONFIG_CNT);
  unsigned int* addr;
  addr = (unsigned int*) C_CURSOR_COLOR_RANGE_BASEADDR;
  *(addr+0) = 0x1D;
  *(addr+1) = 0xFF;
  *(addr+2) = 0x0;
  *(addr+3) = 0x12;
  *(addr+4) = 0x0;
  *(addr+5) = 0x12;
*/

  /* const int cursor_color = 0xFFFFFFFF; // White */
  /* clear_screen(); */

  // Create gestures
  
  // Reset signal to comparison module
  /* simple_write_to_comparison_module(comparison_module_reset_reg, 0x00000000); */
  /* simple_write_to_comparison_module(comparison_module_reset_reg, 0x00000001); */

  while(1)
  {
 //   addr = (unsigned int *) C_CURSOR_LOCATION_BASEADDR;
 //   x = addr[0];
 //   y = addr[1];

    // Write cursor location to test gesture buffer
    //write_to_comparison_module(comparison_module_input_reg, COMP_COMMAND_WRITE_RAW_TEST_BUFFER, 1, x, y);
    *comparison_module_input_reg = (1*COMMAND_OFFSET)+(x*X_OFFSET)+(y*Y_OFFSET)+(1*VALUE);
	 
  }


  ////////////////////////////// Control Loop //////////////////////////////

  unsigned int x_last, y_last, repeat_count; // these are used to detect if cursor doesn't move for some threshold
  unsigned int x_ignore, y_ignore;           // these are used to ignore a stationary cursor at the end of a gesture
  int result1, result2;

  //  xil_printf("Starting parse loop\n\r");

  addr = (unsigned int *) C_CURSOR_LOCATION_BASEADDR;
  x = addr[0];
  y = addr[1];

  x_ignore = x;
  y_ignore = y;

  repeat_count = 0;

   while(1){
    addr = (unsigned int *) C_CURSOR_LOCATION_BASEADDR;
    x = addr[0];
    y = addr[1];

    if (x == x_last && y == y_last && x != x_ignore && y != y_ignore) {
      repeat_count++;
    }
    else {
      x_last = x;
      y_last = y;
      repeat_count = 0;
    }

    // Write cursor location to test gesture buffer
    //write_to_comparison_module(comparison_module_input_reg, COMP_COMMAND_WRITE_RAW_TEST_BUFFER, 1, x, y);
    *comparison_module_input_reg = (1*COMMAND_OFFSET)+(x*X_OFFSET)+(y*Y_OFFSET)+(1*VALUE);
	 
    addr = (unsigned int*) C_VIDEO_RAM_BASEADDR;


    // End of gesture, process this gesture, clear buffers for next one and reset
    if (repeat_count == REPEAT_THRESHOLD) {
      write_1(comparison_module_input_reg);
      //  *comparison_module_input_reg = (9 << COMP_COMMAND_OFFSET);
      *comparison_module_input_reg = (9*COMMAND_OFFSET);
      result1 = wait_until_done(comparison_module_output_reg, comparison_module_input_reg);

      write_2(comparison_module_input_reg);
      /* *comparison_module_input_reg = (9 << COMP_COMMAND_OFFSET); */
      *comparison_module_input_reg = (9*COMMAND_OFFSET);
      result2 = wait_until_done(comparison_module_output_reg, comparison_module_input_reg);
		
      *comparison_module_input_reg = (4*COMMAND_OFFSET);	
      wait_until_done(comparison_module_output_reg,comparison_module_input_reg);
      *comparison_module_input_reg = (5*COMMAND_OFFSET);	
      wait_until_done(comparison_module_output_reg,comparison_module_input_reg);
      *comparison_module_input_reg = (6*COMMAND_OFFSET);	
      wait_until_done(comparison_module_output_reg,comparison_module_input_reg);


      // Setup internal states in preparation for next gesture
      x_ignore = x;
      y_ignore = y;
      repeat_count = 0;
    }
  }
 return 0;
 
 }


