#include <xbasic_types.h>

#include "xparameters.h"
#include "stdio.h"
#include "xutil.h"
#include "xstatus.h"
#include "C:\Users\xli\Desktop\ECE532\integration_3\YAHGC_SW\write_library.h"
#include "C:\Users\xli\Desktop\ECE532\integration_3\YAHGC_SW\buffer_library.h"

#include <xiic_l.h>
#include <math.h>
/* #include <xgpio.h> */

/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////// Global Parameters //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

// Global defines

//#define printf xil_printf

// Controller Defines

#define REPEAT_THRESHOLD 1000

// Push Buttons

#define PUSH_BUTTONS 0x1F /* 0'b11111 */

// Video decoder

#define DECODER_ADDR 0x20 //Read: 0x41, Write: 0x40
#define RED_MASK   0x00003F00
#define GREEN_MASK 0x003F0000
#define BLUE_MASK  0x3F000000

#define RED_SHIFT  8
#define GREEN_SHIFT 16
#define BLUE_SHIFT 24

#define GET_RED_VALUE(raw) (((raw) & RED_MASK) >> RED_SHIFT)
#define GET_GREEN_VALUE(raw) (((raw) & GREEN_MASK) >> GREEN_SHIFT)
#define GET_BLUE_VALUE(raw) (((raw) & BLUE_MASK) >> BLUE_SHIFT)

// Cursor Finder 
// IMPORTANT: manually keep this in sync with cursor_finder_0 instance parameters 

#define C_VIDEO_RAM_BASEADDR          0x40000000
#define C_CURSOR_COLOR_RANGE_BASEADDR 0x42000000
#define C_CURSOR_LOCATION_BASEADDR    0x42001000

// Comparison Module

#define COMMAND_OFFSET 0x08000000
#define X_OFFSET 0x00020000
#define Y_OFFSET 0x00000100
#define VALUE 0x00000001

#define COMP_COMMAND_OFFSET  27
#define COMP_X_OFFSET        17
#define COMP_Y_OFFSET        8
#define COMP_VALUE_OFFSET    0

#define COMP_COMMAND_WRITE_RAW_TEST_BUFFER     1  // write to 640 x 480 x 1 buffer, no ack
#define COMP_COMMAND_WRITE_SCALED_TEST_BUFFER  2  // write to 256 x 256 x 1 buffer, no ack
#define COMP_COMMAND_WRITE_REF_BUFFER          3  // write to 256 x 256 x 8 buffer, no ack
#define COMP_COMMAND_CLEAR_RAW_TEST_BUFFER     4  // clear 640 x 480 x 1 buffer, ack
#define COMP_COMMAND_CLEAR_SCALED_TEST_BUFFER  5  // clear 256 x 256 x 1 buffer, ack
#define COMP_COMMAND_CLEAR_REF_BUFFER          6  // clear 256 x 256 x 8 buffer, ack
#define COMP_COMMAND_COMPARISON_EXECUTE        9  // tells comparison module to do scaling on raw test buffer and compare to ref, ack

// command 19 does scaling only, obsolete
// command 20 used by comparison module to ack when an async command is done

#define COMPARISON_INPUT_REG  0x08000000
#define COMPARISON_OUTPUT_REG 0x08000004
#define COMPARISON_RESET_REG  0x08000010

/////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////// Global Variables //////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

/* unsigned int* comparison_module_input_reg = (unsigned int*)COMPARISON_INPUT_REG; */
/* unsigned int* comparison_module_output_reg = (unsigned int*)COMPARISON_OUTPUT_REG; */
/* unsigned int* comparison_module_reset_reg = (unsigned int*)COMPARISON_RESET_REG; */

/* unsigned int* comparison_module_input_reg =  0x08000000; */
/* unsigned int* comparison_module_output_reg = 0x08000004; */
/* unsigned int* comparison_module_reset_reg =  0x08000010; */

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// VIDEO SETUP /////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

//Composite Configuration constants
struct VideoModule {
  Xuint8 addr;
  Xuint8 config_val;
  Xuint8 actual_val;
};

#define DECODER_COMP_CONFIG_CNT 18
struct VideoModule decoder_comp[] = { 
  { 0x00, 0x04, 0 },
  { 0x15, 0x00, 0 },
  { 0x17, 0x41, 0 },
  { 0x27, 0x58, 0 },
  { 0x3a, 0x16, 0 }, 
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x50, 0x20, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xd7, 0xea, 0 },
  { 0xe4, 0x3e, 0 },
  { 0xea, 0x0f, 0 }, 
  { 0x0e, 0x00, 0 } };

#define DECODER_CMPNT_CONFIG_CNT 13
struct VideoModule decoder_cmpnt[] = { 
  { 0x00, 0x0a, 0 },
  { 0x27, 0xd8, 0 },
  { 0x50, 0x04, 0 },
  { 0x0e, 0x80, 0 },
  { 0x52, 0x18, 0 }, 
  { 0x58, 0xed, 0 },
  { 0x77, 0xc5, 0 },
  { 0x7c, 0x93, 0 },
  { 0x7d, 0x00, 0 },
  { 0xd0, 0x48, 0 },
  { 0xd5, 0xa0, 0 },
  { 0xe4, 0x3e, 0 },
  { 0x0e, 0x00, 0 } };

void configDecoder(struct VideoModule *decoder, int config_cnt );

void configDecoder(struct VideoModule *decoder, int config_cnt )
{
  Xuint16 send_cnt, i;
  Xuint8 send_data[2] = {0};
  Xuint8 success = 1;
  send_cnt = 2;
//  xil_printf("  Configuring Decoder...\r\n");
  //read in configuration constants and send them to the VDEC one by one
  for( i = 0; i < config_cnt; i++ )
    {

      send_data[0] = decoder[i].addr;

      send_data[1] = decoder[i].config_val;

      send_cnt = XIic_Send(XPAR_IIC_BASEADDR, DECODER_ADDR, send_data, 2,XIIC_STOP );

      if( send_cnt != 2 )
        {
  //        xil_printf("Error writing to address %02x\r\n", decoder[i].addr);
          success = 0;
          break;
        }
    }

  if( success ){}
 //   xil_printf("SUCCESS!\r\n");

}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// DRAWING /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

//Writes the given pixel value to the given screen at the given x-y location
void write_pixel(unsigned int value, unsigned int x, unsigned int y)
{
  ( (unsigned int *) C_VIDEO_RAM_BASEADDR)[(y * 1024) + x] = value;
}

void clear_screen()
{
  int x;
  int y;
  for ( y = 0; y < 480; y++)
    {
      for ( x = 0; x < 640; x++)
        {
          write_pixel(0, x, y); // draw black screen
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////// COMPARISON MODULE INTERFACE ////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

#define write_to_comparison_module(reg, command, value, x, y)   \
  *(reg) = (unsigned int)(((command) << COMP_COMMAND_OFFSET) | ((x) << COMP_X_OFFSET) | \
            ((y) << COMP_Y_OFFSET) | ((value) << COMP_VALUE_OFFSET))

#define simple_write_to_comparison_module(reg, value) \
  *(reg) = (unsigned int)(value)

void wait( int wait_time )
{
  int i;
  for ( i = 0; i < wait_time; i++ ){}
  
  return;
}

// 16 possible return values
int default_screen (volatile unsigned int* x_addr, volatile unsigned int* y_addr)
{
	int i, x, y, sw, x_previous, y_previous, x_last, y_last, delay;
	sw = 0;
	
	delay = 0;
	int pixel_value[9];

	write_E(50, 50);
	write_C(57, 50);
	write_E(64, 50);
	write_5(71, 50);
	write_3(78, 50);
	write_2(85, 50);
	
	write_G(109, 50);
	write_E(116, 50);
	write_S(123, 50);
	write_T(130, 50);
	write_U(137, 50);
	write_R(144, 50);
	write_E(151, 50);
	
	write_R(165, 50);
	write_E(172, 50);
	write_C(179, 50);
	write_O(186, 50);
	write_G(193, 50);
	write_N(200, 50);
	write_I(207, 50);
	write_T(214, 50);
	write_I(221, 50);
	write_O(228, 50);
	write_N(235, 50);
	
	for ( i = 0; i < 640; i++ )
		write_a_pixel(i,70);
	
	write_I(50, 80);
	write_N(57, 80);
	write_P(64, 80);
	write_U(71, 80);
	write_T(78, 80);
	
	write_square(50,100);

	write_S(350, 80);
	write_L(357, 80);
	write_I(364, 80);
	write_D(371, 80);
	write_E(378, 80);
	
	write_square(350,100);


	write_V(50, 170);
	write_I(57, 170);
	write_E(64, 170);
	write_W(71, 170);
	
	write_square(50,190);
	write_square(150,190);
	write_square(250,190);
	write_square(350,190);
	write_square(450,190);
	
	write_T(50, 260);
	write_E(57, 260);
	write_A(64, 260);
	write_C(71, 260);
	write_H(78, 260);	
	
	write_square(50,280);
	write_square(150,280);
	write_square(250,280);
	write_square(350,280);
	write_square(450,280);

	write_T(50, 350);
	write_U(57, 350);
	write_N(64, 350);
	write_E(71, 350);
	
	write_square(50,370);
	write_square(150,370);
	write_square(250,370);
	write_square(350,370);
	write_square(450,370);
	
	
	x = *x_addr;
	y = *y_addr;
	x = x &(0x00000fff);
	y = y &(0x00000fff);
	x = 639 - x;
	x_last = x;
	y_last = y;
	
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+0)*Y_OFFSET);
	pixel_value[0] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+0)*Y_OFFSET);
	pixel_value[1] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+0)*Y_OFFSET);
	pixel_value[2] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+1)*Y_OFFSET);
	pixel_value[3] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+1)*Y_OFFSET);
	pixel_value[4] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+1)*Y_OFFSET);
	pixel_value[5] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+2)*Y_OFFSET);
	pixel_value[6] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+2)*Y_OFFSET);
	pixel_value[7] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+2)*Y_OFFSET);
	pixel_value[8] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);

	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
	*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
	
	while ( 1 )
	{
		x_previous = x;
		y_previous = y;
		x = *x_addr;
		y = *y_addr;
		x = x &(0x00000fff);
		y = y &(0x00000fff);
		x = 639 - x;
			
		if ( ( x == x_previous ) && ( y == y_previous ) )
		{
			delay = delay + 1;
		}
		else
		{
			delay = 0;
		}
		
		if ( ( ( x != x_last ) || ( y != y_last ) ) && delay > 10000 )
		{
			delay = 0;
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+0)*X_OFFSET)+((y_last+0)*Y_OFFSET)+(pixel_value[0]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+1)*X_OFFSET)+((y_last+0)*Y_OFFSET)+(pixel_value[1]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+2)*X_OFFSET)+((y_last+0)*Y_OFFSET)+(pixel_value[2]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+0)*X_OFFSET)+((y_last+1)*Y_OFFSET)+(pixel_value[3]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+1)*X_OFFSET)+((y_last+1)*Y_OFFSET)+(pixel_value[4]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+2)*X_OFFSET)+((y_last+1)*Y_OFFSET)+(pixel_value[5]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+0)*X_OFFSET)+((y_last+2)*Y_OFFSET)+(pixel_value[6]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+1)*X_OFFSET)+((y_last+2)*Y_OFFSET)+(pixel_value[7]*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x_last+2)*X_OFFSET)+((y_last+2)*Y_OFFSET)+(pixel_value[8]*VALUE);
			
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+0)*Y_OFFSET);
			pixel_value[0] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+0)*Y_OFFSET);
			pixel_value[1] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+0)*Y_OFFSET);
			pixel_value[2] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+1)*Y_OFFSET);
			pixel_value[3] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+1)*Y_OFFSET);
			pixel_value[4] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+1)*Y_OFFSET);
			pixel_value[5] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+2)*Y_OFFSET);
			pixel_value[6] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+2)*Y_OFFSET);
			pixel_value[7] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			*comparison_module_reg = (7*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+2)*Y_OFFSET);
			pixel_value[8] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
		
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+0)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+1)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+0)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+1)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
			*comparison_module_reg = (1*COMMAND_OFFSET)+((x+2)*X_OFFSET)+((y+2)*Y_OFFSET)+(1*VALUE);
			
			x_last = x;
			y_last = y;
		}
		
		sw = *comparison_module_reg_3;
		
		if (sw == 1)
		{
			if ( check_square(50,100,x,y) )
				return 1;
			else if ( check_square(50,190,x,y) )
				return 2;
			else if ( check_square(150,190,x,y) )
				return 3;
			else if ( check_square(250,190,x,y) )
				return 4;
			else if ( check_square(350,190,x,y) )
				return 5;
			else if ( check_square(450,190,x,y) )
				return 6;
			else if ( check_square(50,280,x,y) )
				return 7;
			else if ( check_square(150,280,x,y) )
				return 8;
			else if ( check_square(250,280,x,y) )
				return 9;
			else if ( check_square(350,280,x,y) )
				return 10;
			else if ( check_square(450,280,x,y) )
				return 11;
			else if ( check_square(50,370,x,y) )
				return 12;
			else if ( check_square(150,370,x,y) )
				return 13;
			else if ( check_square(250,370,x,y) )
				return 14;
			else if ( check_square(350,370,x,y) )
				return 15;
			else if ( check_square(450,370,x,y) )
				return 16;
			else if ( check_square(350,100,x,y) )
				return 17;
		}
	}
}

void parse_gesture(unsigned int* x_addr, unsigned int* y_addr)
{
	int i, j, x, y, x_last, y_last, x_ignore, y_ignore, num_printed, repeat_count;
	int result[5];
	int delay;
	while (*comparison_module_reg_3);

	num_printed = 0;
	repeat_count = 0;
	x = *x_addr;
	y = *y_addr;
	x = x &(0x00000fff);
	y = y &(0x00000fff);
	x = 639 - x;
	
	x_last = x;
	y_last = y;
	x_ignore = x;
	y_ignore = y;
	  
	while(1)
	{
	
	x = *x_addr;
	y = *y_addr;
	x = x &(0x00000fff);
	y = y &(0x00000fff);
	x = 639 - x;
	
	if ( (x != x_ignore) && (y != y_ignore))
	*comparison_module_reg_0 = (1*COMMAND_OFFSET)+(x*X_OFFSET)+(y*OFFSET)+(1*VALUE);
		
	if (x == x_last && y == y_last) {
		repeat_count++;
		delay++;
	}
	else {
		x_last = x;
		y_last = y;
		num_printed = num_printed + 1;
		repeat_count = 0;
		delay = 0;
	}
	  
	if ( repeat_count > 2000000 )
	{
		if ( num_printed < 10)
		{
			for ( i = 0; i < 640; i++ )
			{
				for ( j = 0; j < 480; j++ )
					*comparison_module_reg = (1*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}	
		}
		else
		{
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
					*comparison_module_reg = (2*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
					*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_1();
			*comparison_module_reg_0 = (9*COMMAND_OFFSET);
			result[0] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_2();
			*comparison_module_reg_0 = (9*COMMAND_OFFSET);
			result[1] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_3();
			*comparison_module_reg_0 = (9*COMMAND_OFFSET);
			result[2] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_4();
			*comparison_module_reg_0 = (9*COMMAND_OFFSET);
			result[3] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_5();
			*comparison_module_reg_0 = (9*COMMAND_OFFSET);
			result[4] = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);

			
			*comparison_module_reg_0 = (6*COMMAND_OFFSET);	
			wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	

			for ( i = 20; i < 31; i=i+1 )
			{
				for ( j = (255-result[0]); j < 256; j=j+1 )
				{
					*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*OFFSET)+(255*VALUE);
				}
			}
			for ( i = 71; i < 82; i=i+1 )
			{
				for ( j = (255-result[1]); j < 256; j=j+1 )
				{
					*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*OFFSET)+(255*VALUE);
				}
			}
			for ( i = 122; i < 133; i=i+1 )
			{
				for ( j = (255-result[2]); j < 256; j=j+1 )
				{
					*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*OFFSET)+(255*VALUE);
				}
			}
			for ( i = 173; i < 184; i=i+1 )
			{
				for ( j = (255-result[3]); j < 256; j=j+1 )
				{
					*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*OFFSET)+(255*VALUE);
				}
			}
			for ( i = 224; i < 235; i=i+1 )
			{
				for ( j = (255-result[4]); j < 256; j=j+1 )
				{
					*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*OFFSET)+(255*VALUE);
				}
			}
			return;
		}
		num_printed = 0;
		repeat_count = 0;
		x = x &(0x00000fff);
		y = y &(0x00000fff);
		x = 639 - x;
		y_ignore = y;
		x_ignore = x;

	}
	}
}


/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// MAIN ////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

#define GPO_REG_OFFSET 0x124

#define SEND_CNT 3
#define RECV_CNT 3
#define GPO_RESETS_OFF 1
#define GPO_RESET_IIC 3
#define GPO_RESET_DECODER 0

int main() 
{

  int i, j, result1, result2, x, y, num_printed, x_ignore, y_ignore;
  volatile int sw;
  unsigned int x_last, y_last, repeat_count;       
  *comparison_module_reg_4 = 0x00000000;
  *comparison_module_reg_4 = 0x00000001;
  repeat_count = 0;
  num_printed = 0;
 
  XStatus status;
  
  clear_gesture_1();
  clear_gesture_2();
  clear_gesture_3();
  clear_gesture_4();
  clear_gesture_5();
  initialize_gestures();
  
  configDecoder(decoder_comp, DECODER_COMP_CONFIG_CNT);
  unsigned int* addr;
  unsigned int* addr1;
  unsigned int* addr2;
  addr = (unsigned int*) C_CURSOR_COLOR_RANGE_BASEADDR;
  addr1 = (unsigned int *) C_CURSOR_LOCATION_BASEADDR;
  addr2 = (unsigned int *) (C_CURSOR_LOCATION_BASEADDR+4);
  *(addr+0) = 0x1D;
  *(addr+1) = 0xFF;
  *(addr+2) = 0x0;
  *(addr+3) = 0x12;
  *(addr+4) = 0x0;
  *(addr+5) = 0x12;

  //parse_gesture(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
  //while(1);
  
  
  while(1)
  {
	  for ( i = 0; i < 640; i++ )
	  {
			for ( j = 0; j < 480; j++ )
					*comparison_module_reg = (1*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
	  }
	  result1 = default_screen (C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  
	  if ( result1 == 1 )
	  {
			for ( i = 0; i < 640; i++ )
			{
				for ( j = 0; j < 480; j++ )
					*comparison_module_reg = (1*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			parse_gesture(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 2 )
	  {
	  		for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_1();
	  }
	  else if ( result1 == 3 )
	  {
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_2();
	  }
	  else if ( result1 == 4 )
	  {
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_3();
	  }
	  else if ( result1 == 5 )
	  {
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_4();
	  }
	  else if ( result1 == 6 )
	  {
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_5();
	  }
	  else if ( result1 == 7 )
	  {
			//write_gesture_1(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 8 )
	  {
			//write_gesture_2(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 9 )
	  {
			//write_gesture_3(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 10 )
	  {
			//write_gesture_4(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 11 )
	  {
			//write_gesture_5(C_CURSOR_LOCATION_BASEADDR,(C_CURSOR_LOCATION_BASEADDR+4));
	  }
	  else if ( result1 == 12 )
	  {
	  }
	  else if ( result1 == 13 )
	  {
	  }
	  else if ( result1 == 14 )
	  {
	  }
	  else if ( result1 == 15 )
	  {
	  }
	  else if ( result1 == 16 )
	  {
	  }
	  else if ( result1 == 17 )
	  {	
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_1();
			wait(100000000);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_2();
			wait(100000000);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_3();
			wait(100000000);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_4();
			wait(100000000);
			for ( i = 0; i < 256; i++ )
			{
				for ( j = 0; j < 256; j++ )
				*comparison_module_reg = (3*COMMAND_OFFSET)+((i)*X_OFFSET)+((j)*Y_OFFSET)+(0*VALUE);
			}
			print_gesture_5();
			wait(100000000);
	  }

	  while ( *comparison_module_reg_3);
	}

  return 0;
}



















/*

  while(1);
	  
	  if ( (x != x_ignore) && (y != y_ignore) )
	  *comparison_module_reg_0 = (1*COMMAND_OFFSET)+(x*X_OFFSET)+(y*Y_OFFSET)+(1*VALUE);
		
     if (x == x_last && y == y_last) {
			repeat_count++;
     }
     else {
			x_last = x;
			y_last = y;
			num_printed = num_printed + 1;
			repeat_count = 0;
	  }
	  
	  if ( repeat_count > 1000000 )
	  {
			if ( num_printed < 5)
			{
				*comparison_module_reg_0 = (4*COMMAND_OFFSET);
				wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	
			}
			else
			{
				*comparison_module_reg_0 = (5*COMMAND_OFFSET);	
				wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	
				*comparison_module_reg_0 = (6*COMMAND_OFFSET);	
				wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	
				
				write_gesture_1( comparison_module_reg_0);
				*comparison_module_reg_0 = (9*COMMAND_OFFSET);
				result1 = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
				write_gesture_2( comparison_module_reg_0);
				*comparison_module_reg_0 = (9*COMMAND_OFFSET);
				result2 = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
			
				*comparison_module_reg_0 = (6*COMMAND_OFFSET);	
				wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	

				for ( i = 50; i < 100; i=i+1 )
				{
					for ( j = 50; j < 100; j=j+1 )
					{
						*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*Y_OFFSET)+(result1*VALUE);
					}
				}

				for ( i = 150; i < 200; i=i+1 )
				{
					for ( j = 50; j < 100; j=j+1 )
					{
						*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(j*Y_OFFSET)+(result2*VALUE);
					}
				}
			}
			num_printed = 0;
			repeat_count = 0;
			x = x &(0x00000fff);
			y = y &(0x00000fff);
			y_ignore = y;
			x_ignore = 639 - x;
	  }
	  
  }
*/

