////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Yet Another Human Gesture Controller - Remote PC Monitor Program
//
// Author: Rahat Mahmood
// Date: Apr 5, 2011
//
// This program monitors a character device for commands coming from XUP board running YAHGC system
//
////////////////////////////////////////////////////////////////////////////////////////////////////


#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <string>
#include <sstream>

#include <unistd.h>
#include <sys/wait.h>

#include "common.h"

#define INPUT_BUFFER_SIZE 4096

using namespace std;

// forks a child process which executes kepress task with argv = { ..., task_id, ... }
int fork_task(int task_id) {
  pid_t pid;
  pid = vfork();
  if (pid == 0) { // child process
    stringstream arg;
    arg << task_id;
    char c_arg[10]; // c_string for arg
    strcpy(c_arg, arg.str().c_str());
    char* argv[] = { "./keypress", c_arg, (char*) 0 };
    if (execv("keypress", argv) == -1) {
      cout << "Child process failed to execute task" << endl;
      exit(1);
    }
  }
  else if (pid < 0) {
    cout << "Failed to fork" << endl;
    return 0; // fault
  }
  else { // parent process
#ifdef WAIT_FOR_TASK
    waitpid(pid, NULL, 0);
    cout << "Child process exited, resuming parent execution" << endl;
#endif /* WAIT_FOR_TASK */
    return 1; // success
  }
  return 0;
}

int main(int argc, char* argv[]) {

  ///////////////////////////// Initialization //////////////////////////

  if (argc != 2) {
    cout << "Error: Incorrect number of arguments to Monitor, expect 1, got " <<  argc - 1 << endl;
    exit(1);
  }

  char* dev_file = argv[1];
  cout << "Using device " << dev_file << endl;

  int fd = -1;

#ifdef OPEN_ONCE
  fd = open(dev_file, O_RDONLY, S_IRUSR);
  if (fd < 0) {
    cout << "Failed to open device, Monitor exiting due to fatal error" << endl;
    exit(1);
  }
#endif /* OPEN_ONCE */

  ////////////////////////////// Main Loop //////////////////////////////

  //  for(int i = 0; i <= 0; i++) {

    char data[INPUT_BUFFER_SIZE];    
    char *data_base = data;
    bool more = true;
    int read_count = 0;


  while (true) {

    if (!more) {
      memset(data, 0, INPUT_BUFFER_SIZE);
      more = false;
      read_count = 0;
      data_base = data;
    }

#ifndef OPEN_ONCE
    fd = open(dev_file, O_RDONLY, S_IRUSR);
    if (fd < 0) {
      cout << "Failed to open device, Monitor exiting due to fatal error" << endl;
      exit(1);
    }
#endif /* ~OPEN_ONCE */
    read_count = read(fd,data_base,INPUT_BUFFER_SIZE);
    if (read_count < 0) {
      cout << "Read error" << endl;
    }
    else {
      //      cout << "RAW: " << data_base << endl;
      if ((data_base[read_count - 1] != '\n') && (data_base[read_count - 2] != '\n')) {
        // cout << "moar: " << data_base << endl;
        more = true;
        data_base += read_count;
        continue;
      }
      else {
        more = false;
      }

      string str(data);
      string cmd_prefix("[cmd]");

      size_t result = str.find(cmd_prefix);

      if (result == string::npos) {
        cout << "MESSAGE: " << data;
      }
      else {
        pid_t pid;
        int cmd_num = atoi(str.substr(result+5).c_str());


        //////////////////// Task LUT ////////////////////

        switch(cmd_num) {
        case 1:

          if (fork_task(PRIOR_KEY_COMMAND)) {
            cout << "Forked task for command " << cmd_num << endl;
          }

          cout << "COMMAND: " << cmd_num << ", forked task" << endl;
          break;

        case 25:

          cout << "COMMAND: " << cmd_num << ", nop" << endl;
          break;

        case 2:
        case 45:

          if (fork_task(NEXT_KEY_COMMAND)) {
            cout << "COMMAND: " << cmd_num << ", forked task" << endl;
          }
             
          break;

        default:

          cout << "Unrecognized command, raw command was: " << str.substr(result);
          break;

        }
      }
    }
#ifndef OPEN_ONCE
    if (close(fd) < 0) {
      cout << "Device close failed" << endl;
      exit(1);
    }
#endif /* ~OPEN_ONCE */
  }

  /////////////////////////////// Cleanup //////////////////////////////

  cout << "Closing device" << endl;

  if (close(fd) < 0) {
    cout << "Device close failed" << endl;
    exit(1);
  }

  cout << "Monitor exiting normally" << endl;
  exit(0);
}
