`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    11:17:50 02/07/2011 
// Design Name: 
// Module Name:    comparison_module 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module compare_module_with_buffers(
	// INTERNAL SIGNALS //
	clock_100mhz,
	clock_25mhz,
	resetn,
	input_data,
	output_data,
	
	// VGA SIGNALS//
	VGA_VSYNCH,
	VGA_HSYNCH,
	VGA_OUT_BLANK_Z,
	VGA_COMP_SYNCH,
	VGA_OUT_PIXEL_CLOCK,
	VGA_OUT_RED,
	VGA_OUT_GREEN,
	VGA_OUT_BLUE,
	
	// DEBUG SIGNALS
	debug_in,
	debug_out_0,
	debug_out_1,
	debug_out_2,
	debug_out_3
	);

//// Input Data formats ////
// COMMAND 	[31:27]
// XCOR		[26:17]
// YCOR		[16:8]
//	DATA		[7:0]
	 
// Write to image buffer //
// COMMAND = 5'd1, XCOR, YCOR, DATA[0] used.
// Write to gesture buffer //
// COMMAND = 5'd2, XCOR[24:17], YCOR[15:8], DATA[0] used.
// Write to database buffer //
// COMMAND = 5'd3, XCOR[24:17], YCOR[15:8], DATA[7:0] used.
// Clear image buffer //
// COMMAND = 5'd4, 
// Clear gesture buffer //
// COMMAND = 5'd5, 
// Clear database buffer //
// COMMAND = 5'd6, 


// Do image scaling //
// COMMAND = 5'd9, wait for output_data to ack //	 

// Do Algorithm //
// COMMAND = 5'd19, wait for output_data to ack //	 
	 
// All data is assumed to be on 'clock' clock domain //	 
input clock_100mhz;
input clock_25mhz;
input resetn;
input [31:0] input_data;
input [31:0] debug_in;
output [31:0] output_data;
output [31:0] debug_out_0;
output [31:0] debug_out_1;
output [31:0] debug_out_2;
output [31:0] debug_out_3;
// VGA SIGNALS //
output VGA_VSYNCH, VGA_HSYNCH, VGA_OUT_BLANK_Z, VGA_COMP_SYNCH, VGA_OUT_PIXEL_CLOCK;
output [7:0] VGA_OUT_RED, VGA_OUT_GREEN, VGA_OUT_BLUE;

reg enable_output_data;
reg clear_output_data;
reg [31:0] output_data;
reg [31:0] data_wire;

reg [5:0] state;
reg [5:0] next_state;

reg [19:0] twenty_bit_counter;
reg enable_twenty_bit_counter;
reg clear_twenty_bit_counter;
reg [23:0] twenty_four_bit_accumulator;
reg [23:0] accumulator_input;
reg enable_accumulator;
reg clear_accumulator;
reg [15:0] sixteen_bit_counter;
reg enable_sixteen_bit_counter;
reg clear_sixteen_bit_counter;
reg [15:0] sixteen_bit_counter_2;
reg enable_sixteen_bit_counter_2;
reg clear_sixteen_bit_counter_2;

reg vga_select;
reg [7:0] color_input_r;
reg [7:0] color_input_g;
reg [7:0] color_input_b;

reg [18:0] mem_input_address_buffer_a;
reg mem_input_data_buffer_a;
reg mem_write_en_buffer_a;
reg [15:0] mem_input_address_gesture_a;
reg mem_input_data_gesture_a;
reg mem_write_en_gesture_a;
reg [15:0] mem_input_address_database_a;
reg [7:0] mem_input_data_database_a;
reg mem_write_en_database_a;

reg [8:0] boundary_reg_top;
reg [8:0] boundary_reg_bottom;
reg [9:0] boundary_reg_left;
reg [9:0] boundary_reg_right;
reg enable_boundary_reg_top;
reg enable_boundary_reg_bottom;
reg enable_boundary_reg_left;
reg enable_boundary_reg_right;
reg clear_boundary_regs;

reg [19:0] vertical_offset, horizontal_offset;
reg [23:0] divident_input;
reg [15:0] divisor_input;
reg [23:0] divident_input_2;
reg [15:0] divisor_input_2;

wire [19:0] x_mapped, y_mapped;

wire buffer_memory_out_a;
wire buffer_memory_out_b;
wire gesture_memory_out_a;
wire gesture_memory_out_b;
wire [7:0] database_memory_out_a;
wire [7:0] database_memory_out_b;
wire [15:0] gesture_memory_addr_b;
wire [15:0] database_memory_addr_b;
wire [9:0] xCounter;
wire [8:0] yCounter;
wire [23:0] quotient;
wire [23:0] quotient_2;

wire [9:0] buffer_height;
wire [9:0] buffer_width;

// STATE MACHINE PARAMETERS //
parameter DEFAULT_STATE = 6'd0;
parameter ACCUMULATE_STATE = 6'd1;
parameter DIVIDE_WAIT_STATE = 6'd2;
parameter FIND_BOUNDARY_STATE = 6'd3;
parameter SCALE_STATE = 6'd4;
parameter SCALE_STATE_2 = 6'd5;
parameter SCALE_WAIT_STATE = 6'd6;
parameter CLEAR_IMAGE_BUFFER = 6'd7;
parameter CLEAR_GESTURE_BUFFER = 6'd8;
parameter CLEAR_DATABASE_BUFFER = 6'd9;
parameter CLEAR_IMAGE_BUFFER_SW = 6'd10;
parameter CLEAR_GESTURE_BUFFER_SW = 6'd11;
parameter CLEAR_DATABASE_BUFFER_SW = 6'd12;
parameter WAIT_STATE_SW = 6'd13;
parameter RETRIEVE_1_BIT = 6'd14;
parameter RETRIEVE_1_BIT_LARGE = 6'd15;
parameter RETRIEVE_8_BIT = 6'd16;
//////////////////////////////

assign buffer_height = boundary_reg_bottom - boundary_reg_top ;
assign buffer_width = boundary_reg_right - boundary_reg_left;

wire [31:0] w0, w1, w2, w3;
assign w0 = sixteen_bit_counter - boundary_reg_left;
assign w1 = sixteen_bit_counter_2 - boundary_reg_top;
assign w2 = w0 * 8'd255;
assign w3 = w1 * 8'd255;

assign x_mapped = horizontal_offset + w2;
assign y_mapped = vertical_offset + w3;

//assign x_mapped = w2;
//assign y_mapped = w3;

assign debug_out_0 = state;
assign debug_out_1 = buffer_height;
assign debug_out_2 = buffer_width;
assign debug_out_3 = horizontal_offset;

always @ ( posedge clock_100mhz )
begin
	if ( resetn == 1'b0 )
	begin
		state <= DEFAULT_STATE;
	end
	else
	begin
		state <= next_state;
	end
end

always @ (*)
begin
	case (state)
	DEFAULT_STATE: 
		begin
			if ( debug_in[1] == 1'b1 )
				next_state <= CLEAR_IMAGE_BUFFER_SW;
			else if ( debug_in[2] == 1'b1 )
				next_state <= CLEAR_GESTURE_BUFFER_SW;
			else if ( debug_in[3] == 1'b1 )
				next_state <= CLEAR_DATABASE_BUFFER_SW;
			else if ( input_data[31:27] == 5'd19 )
				next_state <= ACCUMULATE_STATE;
			else if ( input_data[31:27] == 5'd9 )
				next_state <= FIND_BOUNDARY_STATE;
			else if ( input_data[31:27] == 5'd4 )
				next_state <= CLEAR_IMAGE_BUFFER;
			else if ( input_data[31:27] == 5'd5 )
				next_state <= CLEAR_GESTURE_BUFFER;
			else if ( input_data[31:27] == 5'd6 )
				next_state <= CLEAR_DATABASE_BUFFER;
			else if ( input_data[31:27] == 5'd7 )
				next_state <= RETRIEVE_1_BIT_LARGE;
			else if ( input_data[31:27] == 5'd8 )
				next_state <= RETRIEVE_1_BIT;
			else if ( input_data[31:27] == 5'd10 )
				next_state <= RETRIEVE_8_BIT;
			else
				next_state <= DEFAULT_STATE;
		end
	RETRIEVE_1_BIT_LARGE:
		begin
				next_state <= DIVIDE_WAIT_STATE;
		end
	RETRIEVE_1_BIT:
		begin
				next_state <= DIVIDE_WAIT_STATE;
		end
	RETRIEVE_8_BIT:
		begin
				next_state <= DIVIDE_WAIT_STATE;
		end
	CLEAR_IMAGE_BUFFER:
		begin
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
				next_state <= DIVIDE_WAIT_STATE;
			else
				next_state <= CLEAR_IMAGE_BUFFER;
		end
	CLEAR_GESTURE_BUFFER:
		begin
			if ( twenty_bit_counter == 20'h10000 )
				next_state <= DIVIDE_WAIT_STATE;
			else
				next_state <= CLEAR_GESTURE_BUFFER;
		end
	CLEAR_DATABASE_BUFFER:
		begin
			if ( twenty_bit_counter == 20'h10000 )
				next_state <= DIVIDE_WAIT_STATE;
			else
				next_state <= CLEAR_DATABASE_BUFFER;
		end
	CLEAR_IMAGE_BUFFER_SW:
		begin
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
				next_state <= WAIT_STATE_SW;
			else
				next_state <= CLEAR_IMAGE_BUFFER_SW;
		end
	CLEAR_GESTURE_BUFFER_SW:
		begin
			if ( twenty_bit_counter == 20'h10000 )
				next_state <= WAIT_STATE_SW;
			else
				next_state <= CLEAR_GESTURE_BUFFER_SW;
		end
	CLEAR_DATABASE_BUFFER_SW:
		begin
			if ( twenty_bit_counter == 20'h10000 )
				next_state <= WAIT_STATE_SW;
			else
				next_state <= CLEAR_DATABASE_BUFFER_SW;
		end
	WAIT_STATE_SW:
		begin
			if ( debug_in[3:1] == 3'd0 )
				next_state <= DEFAULT_STATE;
			else
				next_state <= WAIT_STATE_SW;
		end
	ACCUMULATE_STATE:
		begin
			if ( twenty_bit_counter == 20'h100ff )
				next_state <= DIVIDE_WAIT_STATE;
			else
				next_state <= ACCUMULATE_STATE;
		end
	DIVIDE_WAIT_STATE:
		begin
			if ( input_data[31:27] == 5'd20 )
				next_state <= DEFAULT_STATE;
			else
				next_state <= DIVIDE_WAIT_STATE;
		end
	FIND_BOUNDARY_STATE:
		begin
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
				next_state <= SCALE_STATE;
			else
				next_state <= FIND_BOUNDARY_STATE;
		end
	SCALE_STATE:
		begin
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
				next_state <= SCALE_WAIT_STATE;
			else if ( buffer_memory_out_a )
				next_state <= SCALE_STATE_2;
			else
				next_state <= SCALE_STATE;
		end
	SCALE_STATE_2:
		begin
			if ( twenty_bit_counter == 20'h000ff )
				next_state <= SCALE_STATE;
			else
				next_state <= SCALE_STATE_2;
		end
	SCALE_WAIT_STATE:
		begin
			next_state <= ACCUMULATE_STATE;
		end
	default: next_state <= DEFAULT_STATE;
	endcase
end

always @ (*)
begin
	mem_input_address_buffer_a <= 19'd0;
	mem_input_data_buffer_a <= 1'b0;
	mem_write_en_buffer_a <= 1'b0;
	mem_input_address_gesture_a <= 15'd0;
	mem_input_data_gesture_a <= 1'b0;
	mem_write_en_gesture_a <= 1'b0;
	mem_input_address_database_a <= 15'd0;
	mem_input_data_database_a <= 8'd0;
	mem_write_en_database_a <= 1'b0;
	enable_output_data <= 1'b0;
	clear_output_data <= 1'b1;
	data_wire <= 32'd0;
	enable_twenty_bit_counter <= 1'b0;
	enable_sixteen_bit_counter <= 1'b0;
	enable_sixteen_bit_counter_2 <= 1'b0;
	enable_accumulator <= 1'b0;
	clear_twenty_bit_counter <= 1'b1;
	clear_sixteen_bit_counter <= 1'b1;
	clear_sixteen_bit_counter_2 <= 1'b1;
	clear_accumulator <= 1'b1;
	enable_boundary_reg_top <= 1'b0;
	enable_boundary_reg_bottom <= 1'b0;
	enable_boundary_reg_left <= 1'b0;
	enable_boundary_reg_right <= 1'b0;
	clear_boundary_regs <= 1'b1;
	accumulator_input <= 24'd0;
	divident_input <= 24'd0;
	divisor_input <= 16'd0;
	divident_input_2 <= 24'd0;
	divisor_input_2 <= 16'd0;
	case (state)
	DEFAULT_STATE: 
		begin
			if ( input_data[31:27] == 5'd1 )
			begin
				mem_input_address_buffer_a <= {input_data[26:17], input_data[16:8]};
				mem_input_data_buffer_a <= input_data[0];
				mem_write_en_buffer_a <= 1'b1;
			end
			else if ( input_data[31:27] == 5'd2 )
			begin
				mem_input_address_gesture_a <= {input_data[24:17], input_data[15:8]};
				mem_input_data_gesture_a <= input_data[0];
				mem_write_en_gesture_a <= 1'b1;
			end
			else if ( input_data[31:27] == 5'd3 )
			begin
				mem_input_address_database_a <= {input_data[24:17], input_data[15:8]};
				mem_input_data_database_a <= input_data[7:0];
				mem_write_en_database_a <= 1'b1;
			end
			else if ( input_data[31:27] == 5'd7 )
			begin
				mem_input_address_buffer_a <= {input_data[26:17], input_data[16:8]};
			end
			else if ( input_data[31:27] == 5'd8 )
			begin
				mem_input_address_gesture_a <= {input_data[24:17], input_data[15:8]};
			end
			else if ( input_data[31:27] == 5'd10 )
			begin
				mem_input_address_database_a <= {input_data[24:17], input_data[15:8]};
			end
		end
	RETRIEVE_1_BIT_LARGE:
		begin
			clear_output_data <= 1'b0;
			enable_output_data <= 1'b1;
			data_wire <= {8'haa,23'd0,buffer_memory_out_a};
		end
	RETRIEVE_1_BIT:
		begin
			clear_output_data <= 1'b0;
			enable_output_data <= 1'b1;
			data_wire <= {8'haa,23'd0,gesture_memory_out_a};
		end
	RETRIEVE_8_BIT:
		begin
			clear_output_data <= 1'b0;
			enable_output_data <= 1'b1;
			data_wire <= {8'haa,16'd0,database_memory_out_a[7:0]};
		end
	CLEAR_IMAGE_BUFFER:
		begin
			enable_sixteen_bit_counter <= 1'b1;
			clear_sixteen_bit_counter <= 1'b0;
			clear_sixteen_bit_counter_2 <= 1'b0;
			mem_input_address_buffer_a <= {sixteen_bit_counter[9:0],sixteen_bit_counter_2[8:0]};
			mem_input_data_buffer_a <= 1'b0;
			mem_write_en_buffer_a <= 1'b1;
			if ( sixteen_bit_counter == 10'd639 )
			begin
				clear_sixteen_bit_counter <= 1'b1;
				enable_sixteen_bit_counter_2 <= 1'b1;
			end
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
			begin			
				clear_output_data <= 1'b0;							
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	CLEAR_GESTURE_BUFFER:
		begin
			clear_twenty_bit_counter <= 1'b0;
			enable_twenty_bit_counter <= 1'b1;
			if ( twenty_bit_counter < 20'h10000 )
			begin
				mem_input_address_gesture_a <= twenty_bit_counter[15:0];
				mem_input_data_gesture_a <= 1'b0;
				mem_write_en_gesture_a <= 1'b1;
			end
			else
			begin
				clear_output_data <= 1'b0;
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	CLEAR_DATABASE_BUFFER:
		begin
			clear_twenty_bit_counter <= 1'b0;
			enable_twenty_bit_counter <= 1'b1;
			if ( twenty_bit_counter < 20'h10000 )
			begin
				mem_input_address_database_a <= twenty_bit_counter[15:0];
				mem_input_data_database_a <= 8'h00;
				mem_write_en_database_a <= 1'b1;
			end
			else
			begin
				clear_output_data <= 1'b0;
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	CLEAR_IMAGE_BUFFER_SW:
		begin
			enable_sixteen_bit_counter <= 1'b1;
			clear_sixteen_bit_counter <= 1'b0;
			clear_sixteen_bit_counter_2 <= 1'b0;
			mem_input_address_buffer_a <= {sixteen_bit_counter[9:0],sixteen_bit_counter_2[8:0]};
			mem_input_data_buffer_a <= 1'b0;
			mem_write_en_buffer_a <= 1'b1;
			if ( sixteen_bit_counter == 10'd639 )
			begin
				clear_sixteen_bit_counter <= 1'b1;
				enable_sixteen_bit_counter_2 <= 1'b1;
			end
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
			begin			
				clear_output_data <= 1'b0;							
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	CLEAR_GESTURE_BUFFER_SW:
		begin
			clear_twenty_bit_counter <= 1'b0;
			enable_twenty_bit_counter <= 1'b1;
			if ( twenty_bit_counter < 20'h10000 )
			begin
				mem_input_address_gesture_a <= twenty_bit_counter[15:0];
				mem_input_data_gesture_a <= 1'b0;
				mem_write_en_gesture_a <= 1'b1;
			end
			else
			begin
				clear_output_data <= 1'b0;
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	CLEAR_DATABASE_BUFFER_SW:
		begin
			clear_twenty_bit_counter <= 1'b0;
			enable_twenty_bit_counter <= 1'b1;
			if ( twenty_bit_counter < 20'h10000 )
			begin
				mem_input_address_database_a <= twenty_bit_counter[15:0];
				mem_input_data_database_a <= 8'h00;
				mem_write_en_database_a <= 1'b1;
			end
			else
			begin
				clear_output_data <= 1'b0;
				enable_output_data <= 1'b1;
				data_wire <= {8'haa,24'd0};
			end
		end
	ACCUMULATE_STATE:
		begin
			enable_twenty_bit_counter <= 1'b1;
			clear_twenty_bit_counter <= 1'b0;
			clear_sixteen_bit_counter <= 1'b0;
			clear_accumulator <= 1'b0;
			clear_output_data <= 1'b0;
			if ( twenty_bit_counter < 20'h10000 )
			begin
				mem_input_address_gesture_a <= twenty_bit_counter[15:0];
				mem_input_address_database_a <= twenty_bit_counter[15:0];
				if ( gesture_memory_out_a == 1'b1 )
				begin
					enable_sixteen_bit_counter <= 1'd1;
					enable_accumulator <= 1'd1;
					accumulator_input <= database_memory_out_a;
				end
			end
			else if ( ( twenty_bit_counter >=  20'h10000 ) )
			begin
				divident_input <= twenty_four_bit_accumulator;
				divisor_input <= sixteen_bit_counter[15:0];
				if ( twenty_bit_counter == 20'h100ff )
				begin
					enable_output_data <= 1'b1;
					data_wire <= {8'haa,quotient[23:0]};
				end
			end
		end
	DIVIDE_WAIT_STATE:
		begin
			clear_twenty_bit_counter <= 1'b0;
			clear_sixteen_bit_counter <= 1'b0;
			clear_accumulator <= 1'b0;
			clear_output_data <= 1'b0;
			divident_input <= twenty_four_bit_accumulator;
			divisor_input <= sixteen_bit_counter[15:0];
			if ( input_data[31:27] == 5'd20 )
			begin
				clear_output_data <= 1'b1;
				data_wire <= 32'd0;				
			end
		end
	WAIT_STATE_SW:
		begin
			clear_twenty_bit_counter <= 1'b0;
			clear_sixteen_bit_counter <= 1'b0;
			clear_accumulator <= 1'b0;
			clear_output_data <= 1'b0;
			divident_input <= twenty_four_bit_accumulator;
			divisor_input <= sixteen_bit_counter[15:0];
			if ( debug_in[3:1] == 3'd0 )
			begin
				clear_output_data <= 1'b1;
				data_wire <= 32'd0;				
			end
		end		
	FIND_BOUNDARY_STATE:
		begin
			clear_boundary_regs <= 1'b0;
			enable_sixteen_bit_counter <= 1'b1;
			clear_sixteen_bit_counter <= 1'b0;
			clear_sixteen_bit_counter_2 <= 1'b0;
			mem_input_address_buffer_a <= {sixteen_bit_counter[9:0],sixteen_bit_counter_2[8:0]};
			if ( sixteen_bit_counter == 10'd639 )
			begin
				clear_sixteen_bit_counter <= 1'b1;
				enable_sixteen_bit_counter_2 <= 1'b1;
			end
			if ( buffer_memory_out_a )
			begin
				if ( sixteen_bit_counter_2[8:0] < boundary_reg_top[8:0] )
				begin
					enable_boundary_reg_top <= 1'b1;
					data_wire[18:10] <= sixteen_bit_counter_2[8:0];
				end
				if ( sixteen_bit_counter_2[8:0] > boundary_reg_bottom[8:0] )
				begin
					enable_boundary_reg_bottom <= 1'b1;
					data_wire[18:10] <= sixteen_bit_counter_2[8:0];
				end
				if ( sixteen_bit_counter[9:0] < boundary_reg_left[9:0] )
				begin
					enable_boundary_reg_left <= 1'b1;
					data_wire[9:0] <= sixteen_bit_counter[9:0];
				end
				if ( sixteen_bit_counter[9:0] > boundary_reg_right[9:0] )
				begin
					enable_boundary_reg_right <= 1'b1;
					data_wire[9:0] <= sixteen_bit_counter[9:0];
				end
			end
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
			begin
				clear_sixteen_bit_counter <= 1'b1;
				clear_sixteen_bit_counter_2 <= 1'b1;
			end
		end
	SCALE_STATE:
		begin
			clear_boundary_regs <= 1'b0;
			clear_sixteen_bit_counter <= 1'b0;
			clear_sixteen_bit_counter_2 <= 1'b0;
			mem_input_address_buffer_a  <= {sixteen_bit_counter[9:0],sixteen_bit_counter_2[8:0]};
			if (!buffer_memory_out_a) 
			begin
				enable_sixteen_bit_counter <= 1'b1;
				if ( sixteen_bit_counter == 16'd639 )
				begin
					enable_sixteen_bit_counter <= 1'b0;
					clear_sixteen_bit_counter <= 1'b1;
					enable_sixteen_bit_counter_2 <= 1'b1;
				end
			end
			if (( sixteen_bit_counter_2[8:0] == 9'd479 ) && ( sixteen_bit_counter[9:0] == 10'd639 ))
			begin
				clear_sixteen_bit_counter <= 1'b1;
				clear_sixteen_bit_counter_2 <= 1'b1;
			end
		end
	SCALE_STATE_2:
		begin
			clear_twenty_bit_counter <= 1'b0;
			clear_boundary_regs <= 1'b0;
			clear_sixteen_bit_counter <= 1'b0;
			clear_sixteen_bit_counter_2 <= 1'b0;
			mem_input_address_buffer_a <= {sixteen_bit_counter[9:0],sixteen_bit_counter_2[8:0]};
			if ( twenty_bit_counter < 20'h000ff )
			begin
				enable_twenty_bit_counter <= 1'b1;
				if ( buffer_height > buffer_width )
				begin
					divident_input <= {12'd0,x_mapped[19:0]};
					divisor_input <= {7'd0,buffer_height[8:0]};
					divident_input_2 <= {12'd0,y_mapped[19:0]};
					divisor_input_2 <= {7'd0,buffer_height[8:0]};
				end
				else if ( buffer_height < buffer_width )
				begin
					divident_input <= {12'd0,x_mapped[19:0]};
					divisor_input <= {6'd0,buffer_width[9:0]};
					divident_input_2 <= {12'd0,y_mapped[19:0]};
					divisor_input_2 <= {6'd0,buffer_width[9:0]};
				end
				else
				begin
					divident_input <= {12'd0,x_mapped[19:0]};
					divisor_input <= {6'd0,buffer_width[9:0]};
					divident_input_2 <= {12'd0,y_mapped[19:0]};
					divisor_input_2 <= {7'd0,buffer_height[8:0]};
				end
			end
			else if ( twenty_bit_counter == 20'h000ff )
			begin
				enable_sixteen_bit_counter <= 1'b1;
				if ( sixteen_bit_counter == 16'd639 )
				begin
					enable_sixteen_bit_counter <= 1'b0;
					clear_sixteen_bit_counter <= 1'b1;
					enable_sixteen_bit_counter_2 <= 1'b1;
				end
				//mem_input_address_gesture_a <= {sixteen_bit_counter[7:0],sixteen_bit_counter_2[7:0]};
				mem_input_address_gesture_a <= {quotient[7:0],quotient_2[7:0]};
				mem_input_data_gesture_a <= 1'b1;
				mem_write_en_gesture_a <= 1'b1;
				divident_input <= {12'd0,x_mapped[19:0]};
				divisor_input <= {6'd0,buffer_width[9:0]};
				divident_input_2 <= {12'd0,y_mapped[19:0]};
				divisor_input_2 <= {7'd0,buffer_height[8:0]};
			end
		end
	SCALE_WAIT_STATE:
		begin
			clear_boundary_regs <= 1'b0;
		end
	default:
		begin
			mem_input_address_buffer_a <= 19'd0;
			mem_input_data_buffer_a <= 1'b0;
			mem_write_en_buffer_a <= 1'b0;
			mem_input_address_gesture_a <= 15'd0;
			mem_input_data_gesture_a <= 1'b0;
			mem_write_en_gesture_a <= 1'b0;
			mem_input_address_database_a <= 15'd0;
			mem_input_data_database_a <= 8'd0;
			mem_write_en_database_a <= 1'b0;
			enable_output_data <= 1'b0;
			clear_output_data <= 1'b1;
			data_wire <= 32'd0;
			enable_twenty_bit_counter <= 1'b0;
			enable_sixteen_bit_counter <= 1'b0;
			enable_sixteen_bit_counter_2 <= 1'b0;
			enable_accumulator <= 1'b0;
			clear_twenty_bit_counter <= 1'b1;
			clear_sixteen_bit_counter <= 1'b1;
			clear_sixteen_bit_counter_2 <= 1'b1;
			clear_accumulator <= 1'b1;
			enable_boundary_reg_top <= 1'b0;
			enable_boundary_reg_bottom <= 1'b0;
			enable_boundary_reg_left <= 1'b0;
			enable_boundary_reg_right <= 1'b0;
			clear_boundary_regs <= 1'b1;
			accumulator_input <= 24'd0;
		end
endcase
end

always @ (*)
begin
	if ( buffer_height > buffer_width )
	begin
		vertical_offset <= 19'd0;
		horizontal_offset <= 8'd127 * ( buffer_height - buffer_width );
	end
	else if( buffer_height < buffer_width )
	begin
		vertical_offset <= 8'd127 * ( buffer_width - buffer_height );
		horizontal_offset <= 19'd0;
	end
	else
	begin
		vertical_offset <= 19'd0;
		horizontal_offset <= 19'd0;
	end
end

/////////////////////////////////
////////Register defines/////////
/////////////////////////////////

// Accumulator Logic 
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		twenty_four_bit_accumulator <= 24'd0;
	else if ( clear_accumulator == 1'b1 )
		twenty_four_bit_accumulator <= 24'd0;
	else if ( enable_accumulator == 1'b1 )
		twenty_four_bit_accumulator <= twenty_four_bit_accumulator + accumulator_input;
end

// Counter logic 20 bit
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		twenty_bit_counter <= 20'd0;
	else if ( clear_twenty_bit_counter == 1'b1 )
		twenty_bit_counter <= 20'd0;
	else if ( enable_twenty_bit_counter == 1'b1 )
		twenty_bit_counter <= twenty_bit_counter + 20'd1;
end

// Counter logic 16 bit
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		sixteen_bit_counter <= 16'd0;
	else if ( clear_sixteen_bit_counter == 1'b1 )
		sixteen_bit_counter <= 16'd0;
	else if ( enable_sixteen_bit_counter == 1'b1 )
		sixteen_bit_counter <= sixteen_bit_counter + 16'd1;
end

// Counter logic 16 bit_2
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		sixteen_bit_counter_2 <= 16'd0;
	else if ( clear_sixteen_bit_counter_2 == 1'b1 )
		sixteen_bit_counter_2 <= 16'd0;
	else if ( enable_sixteen_bit_counter_2 == 1'b1 )
		sixteen_bit_counter_2 <= sixteen_bit_counter_2 + 16'd1;
end

// Output data reg logic
always @ ( posedge clock_100mhz )
begin
	if ( resetn == 1'b0 )
		output_data <= 32'd0;
	else if ( clear_output_data )
		output_data <= 32'd0;
	else if ( enable_output_data )
		output_data <= data_wire;
end

// Boudnary reg storage
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		boundary_reg_top <= 9'd0;
	else if ( clear_boundary_regs )
		boundary_reg_top <= 9'h1ff;
	else if ( enable_boundary_reg_top )
		boundary_reg_top <= data_wire[18:10];
end

// Boudnary reg storage
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		boundary_reg_bottom <= 9'd0;
	else if ( clear_boundary_regs )
		boundary_reg_bottom <= 9'd0;
	else if ( enable_boundary_reg_bottom )
		boundary_reg_bottom <= data_wire[18:10];
end

// Boudnary reg storage
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		boundary_reg_left <= 10'd0;
	else if ( clear_boundary_regs )
		boundary_reg_left <= 10'h3ff;
	else if ( enable_boundary_reg_left )
		boundary_reg_left <= data_wire[9:0];
end

// Boudnary reg storage
always @ ( posedge clock_100mhz )
begin
	if ( !resetn )
		boundary_reg_right <= 10'd0;
	else if ( clear_boundary_regs )
		boundary_reg_right <= 10'd0;
	else if ( enable_boundary_reg_right )
		boundary_reg_right <= data_wire[9:0];
end

////////////////////////////////////////
///////////module instantiation/////////
////////////////////////////////////////

// divider //
divider divider(
	.dividend(divident_input),
	.divisor(divisor_input),
	.quot(quotient),
	.clk(clock_100mhz),
	.remd(),
	.rfd());
	
// divider //
divider divider_2(
	.dividend(divident_input_2),
	.divisor(divisor_input_2),
	.quot(quotient_2),
	.clk(clock_100mhz),
	.remd(),
	.rfd());

// MEMORY FOR VIDEO INPUT 640x480 BUFFER //
memory_1bit_640_480 memory_1bit_640_480(
	// PORT A USED FOR INPUT/OUTPUT //
	.clka(clock_100mhz),
	.dina(mem_input_data_buffer_a),
	.addra(mem_input_address_buffer_a),
	.wea(mem_write_en_buffer_a),
	.douta(buffer_memory_out_a),
	// PORTS B USED FOR OUTPUT ONLY //
	.clkb(clock_25mhz),
	.dinb(1'b0),
	.addrb({xCounter,yCounter}),
	.web(1'b0),
	.doutb(buffer_memory_out_b));

// MEMORY FOR VIDEO INPUT PARSED GESTURE //
memory_1bit_256_256 memory_1bit_256_256_1(
	// PORT A USED FOR INPUT/OUTPUT //
	.clka(clock_100mhz),
	.dina(mem_input_data_gesture_a),
	.addra(mem_input_address_gesture_a),
	.wea(mem_write_en_gesture_a),
	.douta(gesture_memory_out_a),
	// PORTS B USED FOR OUTPUT ONLY //
	.clkb(clock_25mhz),
	.dinb(1'b0),
	.addrb(gesture_memory_addr_b),
	.web(1'b0),
	.doutb(gesture_memory_out_b));

// TRANSLATION FOR PARSED GESTURE MEMORY OUTPUT //
wire [7:0] 	gesture_memory_addr_x, gesture_memory_addr_y;
assign gesture_memory_addr_x = xCounter - 10'd41;
assign gesture_memory_addr_y = yCounter - 9'd112;
assign gesture_memory_addr_b = {gesture_memory_addr_x,gesture_memory_addr_y};	

// MEMORY FOR COMPARSION DATABASE //
memory_8bit_256_256 memory_8bit_256_256_2(
	// PORT A USED FOR INPUT/OUTPUT //
	.clka(clock_100mhz),
	.dina(mem_input_data_database_a),
	.addra(mem_input_address_database_a),	
	.wea(mem_write_en_database_a),
	.douta(database_memory_out_a),
	// PORTS B USED FOR OUTPUT ONLY //
	.clkb(clock_25mhz),
	.dinb(1'b0),
	.addrb(database_memory_addr_b),
	.web(1'b0),
	.doutb(database_memory_out_b));

// TRANSLATION FOR COMPARISON DATABASE MEMORY OUTPUT //
wire [7:0] 	database_memory_addr_x, database_memory_addr_y;
assign database_memory_addr_x = xCounter - 10'd339;
assign database_memory_addr_y = yCounter - 9'd112;
assign database_memory_addr_b = {database_memory_addr_x,database_memory_addr_y};	

//////////////////////////////////
///////////vga logic//////////////
//////////////////////////////////

always @(*)
begin
	if ( debug_in[0] == 1'b0 )
	begin
		if ( ( xCounter >= 42 )&&( xCounter < 296 )&&( yCounter >= 112 )&&( yCounter < 368 ) )
		begin
			// DISPLAY GESTURE MEMORY //
			color_input_r = 8'd0;
			color_input_g = {gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b,gesture_memory_out_b};
			color_input_b = 8'd0;
		end
		else if ( ( xCounter >= 340 )&&( xCounter < 596 )&&( yCounter >= 112 )&&( yCounter < 368 ) )
		begin
			// DISPLAY GESTURE MEMORY //
			color_input_r = {
						database_memory_out_b[0]
						,database_memory_out_b[1]
						,database_memory_out_b[2]
						,database_memory_out_b[3]
						,database_memory_out_b[4]
						,database_memory_out_b[5]
						,database_memory_out_b[6]
						,database_memory_out_b[7]};
			color_input_g = 8'd0;
			color_input_b = 8'd0;
		end
		else
		begin
			// DISPLAY BACKGROUND //
			color_input_r = 8'd255;
			color_input_g = 8'd255;
			color_input_b = 8'd255;
		end
	end
	else if ( debug_in[0] == 1'b1 )
	begin
		if ( ( xCounter >= 0 )&&( xCounter < 640 )&&( yCounter >= 0 )&&( yCounter < 480 ) )
			begin
			// DISPLAY GESTURE MEMORY //
			color_input_r = 8'd0;
			color_input_g = {buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b,buffer_memory_out_b};
			color_input_b = 8'd0;
		end
	end
end

vga_controller vga_controller(
			.resetn(resetn),
			.CLOCK(clock_25mhz),
			.COLOR_INPUT_R(color_input_r),
			.COLOR_INPUT_B(color_input_b),
			.COLOR_INPUT_G(color_input_g),
			.xCounter(xCounter),
			.yCounter(yCounter),
			// OUTPUT VGA SIGNALS //
			.VGA_R(VGA_OUT_RED),
			.VGA_G(VGA_OUT_GREEN),
			.VGA_B(VGA_OUT_BLUE),
			.VGA_HS(VGA_HSYNCH),
			.VGA_VS(VGA_VSYNCH),
			.VGA_BLANK(VGA_OUT_BLANK_Z),
			.VGA_SYNC(VGA_COMP_SYNCH),
			.VGA_CLK(VGA_OUT_PIXEL_CLOCK));

endmodule
