
//--- VGA Adapter Settings
/* The following settings adjust whether the VGA Adapter operates
 * in monochromatic or color mode. If the adapter is to operate in
 * color mode, the developer may specify the range of colors by
 * selecting the width of each color channel (i.e. RED, GREEN, BLUE).
 */

/* Color Modes:
 *	Comment the line below to enable color support 
 */

module vga_controller(
			resetn,
			CLOCK,
			COLOR_INPUT_R,
			COLOR_INPUT_B,
			COLOR_INPUT_G,
			xCounter,
			yCounter,
			
			// OUTPUT VGA SIGNALS //
			VGA_R,
			VGA_G,
			VGA_B,
			VGA_HS,
			VGA_VS,
			VGA_BLANK,
			VGA_SYNC,
			VGA_CLK);

	// CURRENT SETTINGS 640x480 //
	parameter C_VERT_NUM_PIXELS  = 16'd480;
	parameter C_VERT_SYNC_START  = 16'd490;
	parameter C_VERT_SYNC_END    = 16'd492;
	parameter C_VERT_TOTAL_COUNT = 16'd525;
	parameter C_HORZ_NUM_PIXELS  = 16'd640;
	parameter C_HORZ_SYNC_START  = 16'd656;
	parameter C_HORZ_SYNC_END    = 16'd752; 
	parameter C_HORZ_TOTAL_COUNT = 16'd800;

	// MODULE INTERFACE SIGNALS //
	input resetn;
	input CLOCK;
	input [7:0] COLOR_INPUT_R;
	input [7:0] COLOR_INPUT_B;
	input [7:0] COLOR_INPUT_G;
   output reg [9:0] xCounter; 
	output reg [8:0] yCounter; 
	
	// VGA OUTPUT SIGNALS //
	output [7:0] VGA_R;
	output [7:0] VGA_G;
	output [7:0] VGA_B;
	output reg VGA_HS;
	output reg VGA_VS;
	output reg VGA_BLANK;
	output VGA_SYNC;
	output wire VGA_CLK;



   // reg              CLOCK_25;
   // reg [1:0]        clock_counter;

   // always @ (posedge CLOCK or negedge resetn)
   //   begin
   //      if (!resetn)
   //        begin
   //           clock_counter <= 2'b0;
   //        end
   //      else
   //        begin
   //           if (clock_counter == 2'b11)
   //             begin
   //                clock_counter <= 2'b0;
   //                CLOCK_25 <= ~(CLOCK_25);
   //             end
   //           else
   //             begin
   //                clock_counter <= clock_counter + 1'b1;
   //             end
   //        end
   //   end
   
   
	assign VGA_CLK = CLOCK;
   	// assign VGA_CLK = CLOCK_25;
	
	wire	[9:0]	color_out_en;	
	assign  color_out_en = (xCounter < C_HORZ_NUM_PIXELS ) && ( yCounter < C_VERT_NUM_PIXELS);
	
	//- Horizontal Counter
	wire xCounter_clear;
	assign xCounter_clear = (xCounter == (C_HORZ_TOTAL_COUNT-1));

	always @(posedge CLOCK or negedge resetn)
        // always @(posedge CLOCK_25 or negedge resetn)
	begin
		if (!resetn)
			xCounter <= 16'd0;
		else if (xCounter_clear)
			xCounter <= 16'd0;
		else
		begin
			xCounter <= xCounter + 1'b1;
		end
	end
	
	//- Vertical Counter
	wire yCounter_clear;
	assign yCounter_clear = (yCounter == (C_VERT_TOTAL_COUNT-1)); 

	always @(posedge CLOCK or negedge resetn)
  	// always @(posedge CLOCK_25 or negedge resetn)
	begin
		if (!resetn)
			yCounter <= 16'd0;
		else if (xCounter_clear && yCounter_clear)
			yCounter <= 16'd0;
		else if (xCounter_clear)		//Increment when x counter resets
			yCounter <= yCounter + 1'b1;
	end
	
	//--- Output Color

	assign VGA_R[7:0] = (color_out_en)? COLOR_INPUT_R[7:0]:8'd0;
	assign VGA_G[7:0] = (color_out_en)? COLOR_INPUT_G[7:0]:8'd0;
	assign VGA_B[7:0] = (color_out_en)? COLOR_INPUT_B[7:0]:8'd0;

	//--- Output Display Sync and Enable
	/* The following outputs are delayed by two clock cycles total because
	 * of the ram. One cycle is added explicitly below and another cycle
	 * is added on the registered output.
	 */
	
	always @(posedge CLOCK or negedge resetn)
        // always @(posedge CLOCK_25 or negedge resetn)
	begin
		if (!resetn)
		begin
			VGA_HS <= 1'b0;
			VGA_VS <= 1'b0;
			VGA_BLANK <= 1'b0;
		end	
		else
		begin	
			//- Sync Generator (ACTIVE LOW)
			VGA_HS <= ~((xCounter >= C_HORZ_SYNC_START) && (xCounter <= C_HORZ_SYNC_END));
			VGA_VS <= ~((yCounter >= C_VERT_SYNC_START) && (yCounter <= C_VERT_SYNC_END));
			//- Current X and Y is valid pixel range
			VGA_BLANK <= ((xCounter < C_HORZ_NUM_PIXELS) && (yCounter < C_VERT_NUM_PIXELS));	
		end	
	end
	
	assign VGA_SYNC = 1'b1;

endmodule
	