#define COMMAND_OFFSET 0x08000000
#define X_OFFSET 0x00020000
#define Y_OFFSET 0x00000100
#define VALUE 0x00000001
#include <math.h>

// Wait
void wait( int wait_time )
{
	int i;
	for ( i = 0; i < wait_time; i++ ){}
	
	return;
};

// Wait until done
int wait_until_done(int* poll_address, int* ak_address)
{
	int value = *poll_address;
	while ( ( value & 0xff000000 ) != 0xaa000000 )
		value = *poll_address;
		
	*ak_address = (20*COMMAND_OFFSET);
	return ( value & 0x00ffffff );
};


// Gaussian distribution LUT
int gaussian( int delta )
{
	if ( delta < 1 )
		return 255;
	else if ( delta < 2 )
		return 250;
	else if ( delta < 3 )
		return 245;
	else if ( delta < 5 )
		return 230;
	else if ( delta < 7 )
		return 215;
	else if ( delta < 9 )
		return 190;
	else if ( delta < 11 )
		return 170;
	else if ( delta < 14 )
		return 150;
	else if ( delta < 17 )
		return 125;
	else if ( delta < 20 )
		return 100;
	else if ( delta < 24 )
		return 90;
	else if ( delta < 28 )
		return 80;
	else if ( delta < 33 )
		return 70;
	else if ( delta < 38 )
		return 60;
	else if ( delta < 45 )
		return 50;
	else if ( delta < 52 )
		return 40;
	else
		return 0;
}

// Writes a horizontal line
void write_1( unsigned int* address_register)
{
	 int x_value, y_value, value;
	 for ( y_value = 0; y_value < 256; y_value++ )
	 {	
		for ( x_value = 0; x_value < 256; x_value++ )
		{
			value = gaussian(abs( 127- x_value ));
			if ( value >= 0 )
			{
				*address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(value*VALUE);
			}
			else
			{
				*address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(0*VALUE);
			}
		}
	 }
};

// Writes a vertical line
void write_2( unsigned int* address_register)
{
	 int x_value, y_value, value;
	 for ( y_value = 0; y_value < 256; y_value++ )
	 {	
		for ( x_value = 0; x_value < 256; x_value++ )
		{
			value = gaussian(abs( 127 - y_value ));
			*address_register = (3*COMMAND_OFFSET)+(x_value*X_OFFSET)+(y_value*Y_OFFSET)+(value*VALUE);
		}
	 }
};

// 16 possible return values
int default_screen ()
{
	write_E(50, 50);
	write_C(56, 50);
	write_E(62, 50);
	write_5(68, 50);
	write_3(74, 50);
	write_2(80, 50);
	
	write_G(92, 50);
	write_E(98, 50);
	write_S(104, 50);
	write_T(110, 50);
	write_U(116, 50);
	write_R(122, 50);
	write_E(128, 50);
	
	write_R(140, 50);
	write_E(146, 50);
	write_C(152, 50);
	write_O(158, 50);
	write_G(164, 50);
	write_N(170, 50);
	write_I(176, 50);
	write_T(182, 50);
	write_I(188, 50);
	write_O(194, 50);
	write_N(200, 50);
	
	
	
}

int main()
{

	// Command 1: write to 640 by 480 1 bit buffer, no ack required
	// Command 2: write to 256 by 256 1 bit buffer, no ack required
	// Command 3: write to 256 by 256 8 bit buffer, no ack required
	// Command 4: clears 640 by 480 1 bit buffer, ack required
	// Command 5: clears 256 by 256 1 bit buffer, ack required
	// Command 6: clears 256 by 256 8 bit buffer, ack required
	// Command 9: scales the 640 by 480 into the 256 by 256, also does calculation, ack required*
	// Command 19: does calculation, not required, ack required
	// Command 20: ack command, not required*
	// reg_0 : input_data register	
	// reg_1 : output_data register

	int i, j, result1, result2;
   unsigned int* comparison_module_reg_0 = 0x08000000;
   unsigned int* comparison_module_reg_1 = 0x08000004;
	unsigned int* comparison_module_reg_2 = 0x08000008;
	unsigned int* comparison_module_reg_3 = 0x0800000c;
	unsigned int* comparison_module_reg_4 = 0x08000010;

	// Reset line, just put this at the top ///////
	*comparison_module_reg_4 = 0x00000000;
	*comparison_module_reg_4 = 0x00000001;
	////////////////////////////////////////////////

	// This writes to the 640 by 480 1 bit buffer, this is just a example gesture/////
	for ( i = 250; i < 450; i=i+5 )
	{
		*comparison_module_reg_0 = (1*COMMAND_OFFSET)+(i*X_OFFSET)+(20*Y_OFFSET)+(1*VALUE);
		*comparison_module_reg_0 = (1*COMMAND_OFFSET)+(i*X_OFFSET)+(25*Y_OFFSET)+(1*VALUE);
		*comparison_module_reg_0 = (1*COMMAND_OFFSET)+(i*X_OFFSET)+(30*Y_OFFSET)+(1*VALUE);
		*comparison_module_reg_0 = (1*COMMAND_OFFSET)+(i*X_OFFSET)+(35*Y_OFFSET)+(1*VALUE);
	}
	//////////////////////////////////////////////////////////////////////////////////

	// The write_1 functions writes to the 256 by 256 8 bit gesture ///////////////////
	// Writing 9 with command_offset in reg_0 starts the scaling and calculating///////
	// The wait_until_done will acknowledge the finish signal and clear the ending/////
	// The wait_until_done returns a 8 bit number.  (this is none zero only if command == 9///////
	write_1( comparison_module_reg_0);
	*comparison_module_reg_0 = (9*COMMAND_OFFSET);
	result1 = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);

	write_2( comparison_module_reg_0);
	*comparison_module_reg_0 = (9*COMMAND_OFFSET);
	result2 = wait_until_done(comparison_module_reg_1,comparison_module_reg_0);


	// command 4, 5, 6 are the clear buffer functions, they should be followed by a wait_until_done///
	*comparison_module_reg_0 = (4*COMMAND_OFFSET);	
	wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg_0 = (5*COMMAND_OFFSET);	
	wait_until_done(comparison_module_reg_1,comparison_module_reg_0);
	*comparison_module_reg_0 = (6*COMMAND_OFFSET);	
	wait_until_done(comparison_module_reg_1,comparison_module_reg_0);	
	///////////////////////////////////////////////////////////////////////////////////////////////////

	for ( i = 50; i < 55; i=i+1 )
	{
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(20*Y_OFFSET)+(result1*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(21*Y_OFFSET)+(result1*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(22*Y_OFFSET)+(result1*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(23*Y_OFFSET)+(result1*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(24*Y_OFFSET)+(result1*VALUE);
	}

	for ( i = 200; i < 205; i=i+1 )
	{
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(20*Y_OFFSET)+(result2*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(21*Y_OFFSET)+(result2*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(22*Y_OFFSET)+(result2*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(23*Y_OFFSET)+(result2*VALUE);
		*comparison_module_reg_0 = (3*COMMAND_OFFSET)+(i*X_OFFSET)+(24*Y_OFFSET)+(result2*VALUE);
	}	 

}

